function [p,t]=kmg2d(fd,fh,h0,bbox,dg,nr,pfix,varargin)
%KMG2D 2D-mesh generator using signed distance & size functions.
%--------------------------------------------------------------------
% [p,t,be,bn]=kmg2d(fd,fh,h0,bbox,dg,nr,pfix,fparam)
% Input:
% fd,fh : Signed distance and size functions d(x,y) and h(x,y)
% h0 : Reference edge length
% bbox : Boundary box [xmin,ymin; xmax,ymax]
% dg : Type of triangular mesh: dg=1 linear; dg=2 quadratic
% nr : Number of refinements
% pfix : Fixed nodes (node that must appears in the mesh)
% varargin : Additional parameters passed to fd and fh
% Output:
% p : Node coordinates np*2
% t : Triangle vertices nt*3 (linear) or nt*6 (quadratic)
% be : Boundary edges ne*2 (linear) or nt*3 (quadratic)
% bn : Boundary nodes nb*1
%--------------------------------------------------------------------
% (c) 2009, Koko J., ISIMA, koko@isima.fr
%--------------------------------------------------------------------
% Constants
Fscale=1.2; dt=.2;
epsp=.001; epsr=1; deps=sqrt(eps)*h0; geps=.001*h0;
iterMax=5000; mp=5;
if (nargin>=7), npf=size(pfix,1); else pfix=[]; npf=0; end
% Determine initial distribution
p=zeros(10000,2);
p(1,:)=[bbox(1,1),bbox(1,2)]; p(2,:)=[bbox(2,1),bbox(1,2)];
p(3,:)=[bbox(2,1),bbox(2,2)]; p(4,:)=[bbox(1,1),bbox(2,2)];
n=4; [p,n]=qt(fh,p,bbox,h0,n); p=p(1:n,:);
% Remove nodes outside the domain
np0=size(p,1); dp=feval(fd,p,varargin{:});
ii=find(dp<geps); q=p(ii,:);
% Add fixed nodes
if (npf>0), p=setdiff(q,pfix,'rows'); p=[pfix; p];

else, p=q; end
clear q dp
np=size(p,1);
iter=0; ifix=[]; tolp=1; tolr=10^2;
while (iter<iterMax & tolp>epsp)
    iter=iter+1;
    % Delaunay triangluation
    if (tolr>epsr)
        p0=p;
        t=delaunayn(p);
        % Reject triangles with centroid outside the domain
        pm=(p(t(:,1),:)+p(t(:,2),:)+p(t(:,3),:))/3;
        t=t(feval(fd,pm,varargin{:})<-geps,:);
        % Renumber (locally) triangle vertices counter clockwise
        ar=tarea(p,t); it=find(ar<0);
        itt=t(it,2); t(it,2)=t(it,3); t(it,3)=itt;
        % Form all edges without duplication
        eh=[t(:,[1,2]);t(:,[1,3]);t(:,[2,3])];
        [e,ie,je]=unique(sort(eh,2),'rows');
        % Graphical output of the current mesh
        triplot(t,p(:,1),p(:,2)), axis equal,axis off,drawnow, hold off
    end
    % Compute edge lengths & forces
    evec=p(e(:,1),:)-p(e(:,2),:);
    Le=sqrt(sum(evec.^2,2));
    He=feval(fh,(p(e(:,1),:)+p(e(:,2),:))/2,varargin{:});
    L0=He*Fscale*sqrt(sum(Le.^2)/sum(He.^2));
    L=Le./L0; F=(1-L.^2).*exp(-L.^2)./L;
    Fvec=F*[1,1].*evec;
    % Assemble edge forces on nodes
    Fe=full(sparse(e(:,[1,1,2,2]),ones(size(F))*[1,2,1,2],[Fvec,-Fvec],np,2));
    Fe(1:npf,:)=0;
    if (iter>mp*np), Fe(ifix,:)=0; end
    % Move nodes
    p=p+dt*Fe;
    % Bring back outside nodes
    d=feval(fd,p,varargin{:}); io=find(d>0); ii=find(d<-geps);
    ddx=(feval(fd,[p(io,1)+deps,p(io,2)],varargin{:})-d(io))/deps;
    ddy=(feval(fd,[p(io,1),p(io,2)+deps],varargin{:})-d(io))/deps;
    p(io,:)=p(io,:)-[d(io).*ddx,d(io).*ddy];
    % Stopping criteria
    dp=dt*sqrt(sum(Fe.^2,2));

    tolp=max(dp(ii))/h0;
    tolr=max(sqrt(sum((p-p0).^2,2))/h0);
    % check the nodes speed if iter>mp*np
    if (iter>mp*np), ifix=find(dp<dt*epsp); end
    % check the triangle orientation if tolp<epsp
    if (tolp<epsp)
        [ar]=tarea(p,t); if (min(ar)<0), tolp=1; tolr=1.2; end
    end
    fprintf('iteration: %4d tolerance: %10.6f Fixed nodes: %4d\n',...
    iter,tolp,length(ifix)-npf)
end
% Refine mesh & extract boundary edges and nodes
if (dg==1 & nr==0)
    ne=size(e,1); neh=size(eh,1);
    he=full(sparse(je,1,ones(neh,1),ne,1)); ib=find(he==1);
    bn=union(e(ib,1), e(ib,2));
    be=e(ib,:);
else
    if (dg==1),
        for i=1:nr, [p,t,be,bn]=kmg2dref(p,t,dg,fd); end
    else
        for i=1:nr-1, [p,t,be,bn]=kmg2dref(p,t,1,fd); end
        [p,t,be,bn]=kmg2dref(p,t,2,fd);
    end
end
end
