%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%computes stiffness matrix, right hand-side and solution U using P2 finite%
%element  for the equation -div(S*grad(U))=Q                              %
%inputs :                                                                 %
%         Q : sink  source                                                %
%         GammaD : Dirichlet's border                                     %
%         coord : nodes coordinates                                       %
%         nodes : connectivites                                           %
%         S :  hydraulic conductivity  or its decimal logarithm           %
%                                                                         %
%         zD : values of P on Dirichlet border                          %
%         zN : values of Neumann border                                 %
%         boolean : 1 if we use logarithm of hydraulic conductivity       %
%                   0 otherwise                                           %
%outputs :                                                                %
%         K_global : matrix problem                                       %
%         second_term: right-hand side                                    %
%         U : solution                                                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Abdellahi SOUEID AHMED, %
% Abderrahim JARDANI,     %
% Andr REVIL,            %
% 09/24/2012              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [K_global,second_term,U]=Quasi_staticP2(Q,GammaD,coord,nodes,S,zD,zN,boolean)

global gg  %gravity constant
global rho % volumic mass

[ e, te ] = edgelist( coord, nodes);
test = findinterior( coord, e, te ); %interior nodes

GammaD1=(GammaD) &(~test) ; %Dirichlet's points
GammaN1=(~test)&(~GammaD1); % Neumann points

nbN=size(coord,1); %nombre total de noeuds
U=zeros(nbN,1);
f=U;
UNmann=U; %nbN nombre total de noeuds

N_unk=size(find(~GammaD1==1),1); %Number of unknowns
UNmann(~GammaD1)=(1:1:N_unk)';

%evaluates U on Dirichlet's borders
qq=scdm(coord,Q);
for j=1:nbN
    f(j)=qq(j,1);
    if GammaD1(j)
        U(j)=zD(j);
    end
end

%initialization
K_global=sparse(N_unk,N_unk); second_term=zeros(N_unk,1);

centers=center(coord,nodes);
if boolean==1
    kappa=10.^(-S);
    kappa=griddata(centers(:,1),centers(:,2),kappa(:),coord(:,1),coord(:,2),'nearest');
else
    kappa=S;
    kappa=griddata(centers(:,1),centers(:,2),kappa(:),coord(:,1),coord(:,2),'nearest');
end

%values os integration of shape shape function products
int1=[  2.00000000e-01  -3.33333333e-02  -3.33333333e-02   0.00000000e+00 -1.33333333e-01  -1.66533454e-16];
int2=[ -3.33333333e-02   2.00000000e-01  -3.33333333e-02   0.00000000e+00 0.00000000e+00  -1.33333333e-01];
int3=[ -3.33333333e-02  -3.33333333e-02   2.00000000e-01  -1.33333333e-01 0.00000000e+00  -1.66533454e-16];
int4=[  0.00000000e+00   0.00000000e+00  -1.33333333e-01   1.06666667e+00  5.33333333e-01   5.33333333e-01];
int5=[ -1.33333333e-01   0.00000000e+00   0.00000000e+00   5.33333333e-01 1.06666667e+00   5.33333333e-01];
int6= [ -1.66533454e-16  -1.33333333e-01  -1.66533454e-16   5.33333333e-01 5.33333333e-01   1.06666667e+00];

%loop over elements
for iel=1:size(nodes,1)
    %(j,k,l) : element en question
    j=nodes(iel,1); k=nodes(iel,2); l=nodes(iel,3); jk=nodes(iel,4);
    kl=nodes(iel,5); jl=nodes(iel,6);

    %UNmann(a)>0 if a is unknown
    jj=UNmann(j); kk=UNmann(k); ll=UNmann(l); jjkk=UNmann(jk); jjll=UNmann(jl); kkll=UNmann(kl);

    belem=[f(j) f(k) f(l) f(jk) f(kl) f(jl)];

    x21 = coord(k,1) - coord(j,1);
    x31 = coord(l,1) - coord(j,1);
    y21 = coord(k,2) - coord(j,2);
    y31 = coord(l,2) - coord(j,2);
    jacobian = x21*y31 - x31*y21;
    alpha = (x31^2 + y31^2)/jacobian;
    beta = -(x31*x21 + y31*y21)/jacobian;
    gamma = (x21^2 + y21^2)/jacobian;

    %elementary stiffness matrix
    Kel= alpha/6*[ 3  1  0 -4  0  0;1  3  0 -4  0  0; 0  0  0  0  0  0; ...
        -4 -4  0  8  0  0; 0  0  0  0  8 -8; 0  0  0  0 -8  8]...
        +beta/6*[ 6  1  1 -4  0 -4; 1  0 -1 -4  4  0; 1 -1  0  0  4 -4; ...
        -4 -4  0  8 -8  8; 0  4  4 -8  8 -8; -4  0 -4  8 -8  8]...
        +gamma/6*[ 3  0  1  0  0 -4; 0  0  0  0  0  0; 1  0  3  0  0 -4; ...
        0  0  0  8 -8  0; 0  0  0 -8  8  0; -4  0 -4  0  0  8];

    if boolean==1
        Kel=(10^(-S(iel))/(gg*rho))*Kel;
    else
        Kel=S(iel)*Kel;
    end

    %Assembling global matrix
    if jj>0
        K_global(jj,jj)=K_global(jj,jj)+Kel(1,1);
        if (kk>0)
            K_global(jj,kk)=K_global(jj,kk)+Kel(1,2);
            K_global(kk,jj)=K_global(jj,kk);% matrice symetrique
        end
        if (ll>0)
            K_global(jj,ll)=K_global(jj,ll)+Kel(1,3);
            K_global(ll,jj)=K_global(jj,ll);
        end
        if (jjkk>0)
            K_global(jj,jjkk)=K_global(jj,jjkk)+Kel(1,4);
            K_global(jjkk,jj)=K_global(jj,jjkk);
        end
        if (jjll>0)
            K_global(jj,jjll)=K_global(jj,jjll)+Kel(1,6);
            K_global(jjll,jj)=K_global(jj,jjll);
        end
        if (kkll>0)
            K_global(jj,kkll)=K_global(jj,kkll)+Kel(1,5);
            K_global(kkll,jj)=K_global(jj,kkll);
        end
    end

    if kk>0
        K_global(kk,kk)=K_global(kk,kk)+Kel(2,2);
        if ll>0
            K_global(kk,ll)=K_global(kk,ll)+Kel(2,3);
            K_global(ll,kk)=K_global(kk,ll);
        end
        if jjkk>0
            K_global(kk,jjkk)= K_global(kk,jjkk)+Kel(2,4);
            K_global(jjkk,kk)=K_global(kk,jjkk);
        end
        if jjll>0
            K_global(kk,jjll)= K_global(kk,jjll)+Kel(2,6);
            K_global(jjll,kk)=K_global(kk,jjll);
        end
        if kkll>0
            K_global(kk,kkll)= K_global(kk,kkll)+Kel(2,5);
            K_global(kkll,kk)=K_global(kk,kkll);
        end
    end

    if ll>0
        K_global(ll,ll)=K_global(ll,ll)+Kel(3,3);
        if jjkk>0
            K_global(ll,jjkk)= K_global(ll,jjkk)+Kel(3,4);
            K_global(jjkk,ll)=K_global(ll,jjkk);
        end
        if jjll>0
            K_global(ll,jjll)= K_global(ll,jjll)+Kel(3,6);
            K_global(jjll,ll)=K_global(ll,jjll);
        end
        if kkll>0
            K_global(ll,kkll)= K_global(ll,kkll)+Kel(3,5);
            K_global(kkll,ll)=K_global(ll,kkll);
        end
    end

    if jjkk>0
        K_global(jjkk,jjkk)=K_global(jjkk,jjkk)+Kel(4,4);
        if (jjll>0)
            K_global(jjkk,jjll)=K_global(jjkk,jjll)+Kel(4,6);
            K_global(jjll,jjkk)=K_global(jjkk,jjll);% matrice symetrique
        end
        if (kkll>0)
            K_global(jjkk,kkll)=K_global(jjkk,kkll)+Kel(4,5);
            K_global(kkll,jjkk)=K_global(jjkk,kkll);
        end
    end

    if kkll>0
        K_global(kkll,kkll)= K_global(kkll,kkll)+Kel(5,5);
        if jjll>0
            K_global(kkll,jjll)= K_global(kkll,jjll)+Kel(6,5);
            K_global(jjll,kkll)=K_global(kkll,jjll);
        end
    end

    if jjll>0
        K_global(jjll,jjll)= K_global(jjll,jjll)+Kel(6,6);
    end

    cste=jacobian/12;
    %Computing right-hande side

    % f is projected on shape functions basis
    if jj>0
        second_term(jj)=second_term(jj)+cste*belem*int1';
    else
        if kk>0
            %part of b involving stiffness matrix terms
            second_term(kk)=second_term(kk)-U(j)*Kel(1,2);
        end
        if ll>0
            second_term(ll)=second_term(ll)-U(j)*Kel(1,3);
        end
        if jjkk>0
            second_term(jjkk)=second_term(jjkk)-U(j)*Kel(1,4);
        end
        if jjll>0
            second_term(jjll)=second_term(jjll)-U(j)*Kel(1,6);
        end
        if kkll>0
            second_term(kkll)=second_term(kkll)-U(j)*Kel(1,5);
        end
    end

    if kk>0
        second_term(kk)=second_term(kk)+cste*belem*int2';
    else
        if jj>0
            second_term(jj)=second_term(jj)-U(k)*Kel(1,2);
        end
        if ll>0
            second_term(ll)=second_term(ll)-U(k)*Kel(2,3);
        end
        if jjkk>0
            second_term(jjkk)=second_term(jjkk)-U(k)*Kel(2,4);
        end
        if jjll>0
            second_term(jjll)=second_term(jjll)-U(k)*Kel(2,6);
        end
        if kkll>0
            second_term(kkll)=second_term(kkll)-U(k)*Kel(2,5);
        end
    end

    if ll>0
        second_term(ll)=second_term(ll)+cste*belem*int3';
    else
        if jj>0
            second_term(jj)=second_term(jj)-U(l)*Kel(1,3);
        end
        if kk>0
            second_term(kk)=second_term(kk)-U(l)*Kel(2,3);
        end
        if jjkk>0
            second_term(jjkk)=second_term(jjkk)-U(l)*Kel(3,4);
        end,
        if jjll>0
            second_term(jjll)=second_term(jjll)-U(l)*Kel(3,6);
        end
        if kkll>0
            second_term(kkll)=second_term(kkll)-U(l)*Kel(3,5);
        end
    end

    if jjkk>0
        second_term(jjkk)=second_term(jjkk)+cste*belem*int4';
    else
        if jj>0
            second_term(jj)=second_term(jj)-U(jk)*Kel(4,1);
        end
        if kk>0
            second_term(kk)=second_term(kk)-U(jk)*Kel(4,2);
        end
        if ll>0
            second_term(ll)=second_term(ll)-U(jk)*Kel(4,3);
        end
        if jjll>0
            second_term(jjll)=second_term(jjll)-U(jk)*Kel(4,6);
        end
        if kkll>0
            second_term(kkll)=second_term(kkll)-U(jk)*Kel(4,5);
        end
    end

    if kkll>0
        second_term(kkll)=second_term(kkll)+cste*belem*int5';
    else
        if jj>0,
            second_term(jj)=second_term(jj)-U(kl)*Kel(5,1);
        end
        if kk>0,
            second_term(kk)=second_term(kk)-U(kl)*Kel(5,2);
        end
        if ll>0
            second_term(ll)=second_term(ll)-U(kl)*Kel(5,3);
        end,
        if jjkk>0
            second_term(jjkk)=second_term(jjkk)-U(kl)*Kel(5,4);
        end,
        if jjll>0
            second_term(jjll)=second_term(jjll)-U(kl)*Kel(5,6);
        end
    end

    if jjll>0
        second_term(jjll)=second_term(jjll)+cste*belem*int6';
    else
        if jj>0,
            second_term(jj)=second_term(jj)-U(jl)*Kel(6,1);
        end
        if kk>0,
            second_term(kk)=second_term(kk)-U(jl)*Kel(6,2);
        end
        if ll>0
            second_term(ll)=second_term(ll)-U(jl)*Kel(6,3);
        end,
        if jjkk>0
            second_term(jjkk)=second_term(jjkk)-U(jl)*Kel(6,4);
        end,
        if kkll>0
            second_term(kkll)=second_term(kkll)-U(jl)*Kel(6,5);
        end
    end

    %in case we have Neumann contributions, we have to add its contributions
    if GammaN1(j)% in Neumann's border
        if test(k)==0,  %on boundaries
            %1D shape functions evaluation on  GammmaN
            second_term(jj)=second_term(jj)+Neumann_borderP2(coord(j,:),coord(k,:),zN(j),1);
            second_term(jjkk)=second_term(jjkk)+Neumann_borderP2(coord(j,:),coord(k,:),zN(j),2);
        end
        if test(l)==0,
            second_term(jj)=second_term(jj)+Neumann_borderP2(coord(j,:),coord(l,:),zN(j),1);
            second_term(jjll)=second_term(jjll)+Neumann_borderP2(coord(j,:),coord(l,:),zN(j),2);
        end
    end

    if GammaN1(k)
        if test(j)==0,
            second_term(kk)=second_term(kk)+Neumann_borderP2(coord(k,:),coord(j,:),zN(k),1);
        end
        if test(l)==0,
            second_term(kk)=second_term(kk)+Neumann_borderP2(coord(k,:),coord(l,:),zN(k),1);
            second_term(kkll)=second_term(kkll)+Neumann_borderP2(coord(k,:),coord(l,:),zN(k),2);
        end
    end

    if GammaN1(l)
        if test(j)==0,
            second_term(ll)=second_term(ll)+Neumann_borderP2(coord(l,:),coord(j,:),zN(l),1);
        end
        if test(k)==0,
            second_term(ll)=second_term(ll)+Neumann_borderP2(coord(l,:),coord(k,:),zN(l),1);
        end

    end

end
U(~GammaD1)=K_global\second_term;%solution of the problem

