
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%computes stiffness matrix, right hand-side and solution U   for the     %
% equation S*dU/dt-div(S*grad(U))=Q                                      %
%inputs :                                                               %
%         secondM : sink source function
%         s : storage coefficient
%         GammaD : Dirichlet's border                                   %
%         coord : nodes coordinates                                     %
%         nodes : connectivites                                         %
%         S :  hydraulic conductivity  or its decimal logarithm         %
%         Tmax : time of simulation
%         n : number of time steps
%         zD : values of P on Dirichlet border                        %
%         zN : values of Neumann border                               %
%         boolean : 1 if we use logarithm of hydraulic conductivity     %
%                   0 otherwise                                         %
%outputs :                                                              %
%         U : time-dependent solution                                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Abdellahi SOUEID AHMED, %
% Abderrahim JARDANI,     %
% Andr REVIL,            %
% 09/24/2012              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [A,second_term,U]=Time_dependent(S,s,secondM,GammaD,coord,nodes,zD,zN,Tmax,n,boolean,r1,r2)

global gg  %gravity constant
global rho % volumic mass

[ e, te ] = edgelist( coord, nodes);
test = findinterior( coord, e, te ); %interior nodes

GammaD1=(GammaD) &(~test) ; %Dirichlet's points
GammaN1=(~test)&(~GammaD1); % Neumann points

nbN=size(coord,1); %nombre total de noeuds
U=zeros(nbN,n+1);
f=U;
UNmann=U; %nbN nombre total de noeuds

N_unk=size(find(~GammaD1==1),1); %Number of unknowns
UNmann(~GammaD1)=(1:1:N_unk)';




qq=secondM(coord,0);
Ud=zeros(nbN,1); 
%evaluates U on Dirichlet's borders
for j=1:nbN
    f(j)=qq(j,1);
    if GammaD1(j)
        Ud(j)=zD(j);
    end
end
for j=1:n+1
    U(:,j)=Ud(:);
end

%initialization of mass global matrix and stiffness global matrix
Mgl=sparse(N_unk,N_unk);  Kgl=sparse(N_unk,N_unk);

centers=center(coord,nodes);
if boolean==1
    kappa=(10.^(-S)/(9.82*1000));
    kappa=griddata(centers(:,1),centers(:,2),kappa(:),coord(:,1),coord(:,2),'nearest');
else
    kappa=S;
    kappa=griddata(centers(:,1),centers(:,2),kappa(:),coord(:,1),coord(:,2),'nearest');
end


%loop over elements
for iel=1:size(nodes,1)
    %(j,k,l) : element in question
    j=nodes(iel,1); k=nodes(iel,2); l=nodes(iel,3);

    %UNmann(a)>0 if a is unknown
    jj=UNmann(j); kk=UNmann(k); ll=UNmann(l);

    % coordinates of each vertex
    x11=coord(j,1);
    x12=coord(j,2);
    x21=coord(k,1);
    x22=coord(k,2);
    x31=coord(l,1);
    x32=coord(l,2);

    %jacobian(determinant of jacobian matrix)
    jac=abs((x11-x31).*(x32-x22)-(x12-x32).*(x31-x21));

    %gadients of basis functions phii,phij,phik

    %gadient of phii
    gphiix=-(x32-x22)./jac;
    gphiiy=(x31-x21)./jac;
    dphi=[gphiix ; gphiiy];

    %gadient of phij
    gphijx=-(x12-x32)./jac;
    gphijy=(x11-x31)./jac;
    dphij=[gphijx ; gphijy];

    %gadient of phik
    gphikx=-(x22-x12)./jac;
    gphiky=(x21-x11)./jac;
    dphik=[gphikx ; gphiky];

    if nargin == 11
        %elementary stiffness matrix
        k11=dot(dphi,dphi);
        k12=dot(dphi,dphij);
        k13=dot(dphi,dphik);
        k22=dot(dphij,dphij);
        k23=dot(dphij,dphik);
        k33=dot(dphik,dphik);

        %symetry
        k21=k12;
        k31=k13;
        k32=k23;

        if boolean==1
            Kel=0.5*jac*(10^(-S(iel))/(gg*rho))*[k11 k12 k13 ; k21 k22 k23 ; k31 k32 k33];
        else
            Kel=0.5*jac*S(iel)*[k11 k12 k13 ; k21 k22 k23 ; k31 k32 k33];
        end
    else

        k11=r1*gphiix*gphiix+r2*gphiiy*gphiiy;
        k12=r1*gphiix* gphijx+r2*gphiiy*gphijy;
        k13=r1*gphiix* gphikx+r2*gphiiy*gphiky;
        k22=r1*gphijx* gphijx+r2*gphijy*gphijy;
        k23=r1*gphijx* gphikx+r2*gphijy*gphiky;
        k33=r1*gphikx* gphikx+r2*gphiky*gphiky;

        %symetry
        k21=k12;
        k31=k13;
        k32=k23;

        if boolean==1
            Kel=0.5*jac*(10^(-S(iel))/(gg*rho))*[k11 k12 k13 ; k21 k22 k23 ; k31 k32 k33];
        else
            Kel=0.5*jac*S(iel)*[k11 k12 k13 ; k21 k22 k23 ; k31 k32 k33];
        end
    end

    %elementary mass matrix
    Mel=s(iel)*(jac/12)*[1 .5 .5 ; .5 1 .5 ;.5 .5 1];

    %Assembly of global matrices
    if jj>0
        Kgl(jj,jj)=Kgl(jj,jj)+Kel(1,1);
        Mgl(jj,jj)=Mgl(jj,jj)+Mel(1,1);
        if (kk>0)
            Kgl(jj,kk)=Kgl(jj,kk)+Kel(1,2);
            Mgl(jj,kk)=Mgl(jj,kk)+Mel(1,2);
            Kgl(kk,jj)=Kgl(jj,kk);% symetric matrix
            Mgl(kk,jj)=Mgl(jj,kk);% symetric matrix
        end
        if (ll>0)
            Kgl(jj,ll)=Kgl(jj,ll)+Kel(1,3);
            Mgl(jj,ll)=Mgl(jj,ll)+Mel(1,3);
            Kgl(ll,jj)=Kgl(jj,ll);
            Mgl(ll,jj)=Mgl(jj,ll);
        end
    end

    if kk>0
        Kgl(kk,kk)=Kgl(kk,kk)+Kel(2,2);
        Mgl(kk,kk)=Mgl(kk,kk)+Mel(2,2);
        if ll>0
            Kgl(kk,ll)=Kgl(kk,ll)+Kel(2,3);
            Mgl(kk,ll)=Mgl(kk,ll)+Mel(2,3);
            Kgl(ll,kk)=Kgl(kk,ll);
            Mgl(ll,kk)=Mgl(kk,ll);
        end
    end

    if ll>0
        Kgl(ll,ll)=Kgl(ll,ll)+Kel(3,3);
        Mgl(ll,ll)=Mgl(ll,ll)+Mel(3,3);
    end

end

%change size of global matrices to take account for boundary conditions
indices=find(GammaD1==1);
ind=setdiff([1:nbN] , indices);

AA=zeros(nbN,nbN);
for i=1:length(indices)
    AA(indices(i),:)=0;
    AA(indices(i),indices(i))=1;
end
com=1;
for i=1:nbN
    if (AA(i,:)==zeros(1,nbN))==ones(1,nbN)
        AA(i,ind)=Kgl(com,:);
        com=com+1;
    end
end

BB=zeros(nbN,nbN);
for i=1:length(indices)
    BB(indices(i),:)=0;
    BB(indices(i),indices(i))=1;
end
com=1;
for i=1:nbN
    if (BB(i,:)==zeros(1,nbN))==ones(1,nbN)
        BB(i,ind)=Mgl(com,:);
        com=com+1;
    end
end

% AA=full(AA);
% BB=full(BB);
%theta-scheme
theta=1;
tt=0; %initial time
dt = Tmax/n; %time step

A=BB+dt*theta*AA;
B=BB-dt*(1-theta)*AA;


[LA,UA]=lu(A);% LU factorization of A


Fn=zeros(nbN,1); %initialization of right-hand side

%loop over time
for i=1:n
        
    second_term=zeros(N_unk,1);
    tt = tt + dt; % time updating
    f=secondM(coord,tt);

    %loop over elements
    for iel=1:size(nodes,1)
        %(j,k,l) : element in question
        j=nodes(iel,1); k=nodes(iel,2); l=nodes(iel,3);

        %UNmann(a)>0 if a is unknown
        jj=UNmann(j); kk=UNmann(k); ll=UNmann(l);

        % coordinates of each vertex
        x11=coord(j,1);
        x12=coord(j,2);
        x21=coord(k,1);
        x22=coord(k,2);
        x31=coord(l,1);
        x32=coord(l,2);

        %jacobian(determinant of jacobian matrix)
        jac=abs((x11-x31).*(x32-x22)-(x12-x32).*(x31-x21));

        %gadients of basis functions phii,phij,phik

        %gadient of phii
        gphiix=-(x32-x22)./jac;
        gphiiy=(x31-x21)./jac;
        dphi=[gphiix ; gphiiy];

        %gadient of phij
        gphijx=-(x12-x32)./jac;
        gphijy=(x11-x31)./jac;
        dphij=[gphijx ; gphijy];

        %gadient of phik
        gphikx=-(x22-x12)./jac;
        gphiky=(x21-x11)./jac;
        dphik=[gphikx ; gphiky];


        if nargin == 11
            %elementary stiffness matrix
            k11=dot(dphi,dphi);
            k12=dot(dphi,dphij);
            k13=dot(dphi,dphik);
            k22=dot(dphij,dphij);
            k23=dot(dphij,dphik);
            k33=dot(dphik,dphik);

            %symetry
            k21=k12;
            k31=k13;
            k32=k23;



            if boolean==1
                Kel=0.5*jac*(10^(-S(iel))/(gg*rho))*[k11 k12 k13 ; k21 k22 k23 ; k31 k32 k33];
            else
                Kel=0.5*jac*S(iel)*[k11 k12 k13 ; k21 k22 k23 ; k31 k32 k33];
            end
        else

            k11=r1*gphiix*gphiix+r2*gphiiy*gphiiy;
            k12=r1*gphiix* gphijx+r2*gphiiy*gphijy;
            k13=r1*gphiix* gphikx+r2*gphiiy*gphiky;
            k22=r1*gphijx* gphijx+r2*gphijy*gphijy;
            k23=r1*gphijx* gphikx+r2*gphijy*gphiky;
            k33=r1*gphikx* gphikx+r2*gphiky*gphiky;

            %symetry
            k21=k12;
            k31=k13;
            k32=k23;

            if boolean==1
                Kel=0.5*jac*(10^(-S(iel))/(gg*rho))*[k11 k12 k13 ; k21 k22 k23 ; k31 k32 k33];
            else
                Kel=0.5*jac*S(iel)*[k11 k12 k13 ; k21 k22 k23 ; k31 k32 k33];
            end
        end


        belem=[f(j,1) f(k,1) f(l,1)];


        cste=jac/24;

        % f is projected on basis of (phii, phij,phik)
        if jj>0
           second_term(jj)=second_term(jj)+cste*belem*[2;1;1];
        else
            if kk>0
                %part of b involving stifness matrix terms
                second_term(kk)=second_term(kk)-U(j,i)*Kel(1,2);
            end
            if ll>0
                second_term(ll)=second_term(ll)-U(j,i)*Kel(1,3);
            end
        end

        if kk>0
            second_term(kk)=second_term(kk)+cste*belem*[1;2;1];
        else
            if jj>0,
               second_term(jj)=second_term(jj)-U(k,i)*Kel(1,2);
            end
            if ll>0,
                second_term(ll)=second_term(ll)-U(k,i)*Kel(2,3);
            end
        end

        if ll>0
            second_term(ll)=second_term(ll)+cste*belem*[1;1;2];
        else
            if jj>0,
               second_term(jj)=second_term(jj)-U(l,i)*Kel(1,3);
            end
            if kk>0,
                second_term(kk)=second_term(kk)-U(l,i)*Kel(2,3);
            end
        end

        %in case we have Neumann contributions, we have to add its contributions
        if GammaN1(j)% in Neumann's border
            if test(k)==0, %on boundaries
                %1D shape fucntions evaluation on  GammmaN
               second_term(jj)=second_term(jj)+bordN(coord(j,:),coord(k,:),zN(j));
            end
            if test(l)==0,
               second_term(jj)=second_term(jj)+bordN(coord(j,:),coord(l,:),zN(j));
            end
        end

        if GammaN1(k)
            if test(j)==0,
               second_term(kk)=second_term(kk)+bordN(coord(k,:),coord(j,:),zN(k));
            end
            if test(l)==0,
               second_term(kk)=second_term(kk)+bordN(coord(k,:),coord(l,:),zN(k));
            end
        end

        if GammaN1(l)
            if test(j)==0,
                second_term(ll)=second_term(ll)+bordN(coord(l,:),coord(j,:),zN(l));
            end
            if test(k)==0,
                second_term(ll)=second_term(ll)+bordN(coord(l,:),coord(k,:),zN(l));
            end
        end

    end

    bb=zeros(nbN,1);%initialization of right-hand side
    bb(~GammaD1)=second_term;

    %values on Dirichlet borders are already known
    bb(GammaD1)=Ud(GammaD1);

    Fnp1=bb;
    F=dt*((1-theta)*Fn+theta*Fnp1);
    Fn=Fnp1;%updates theta-scheme

    %solves problem
    Y=B*U(:,i)+F;
    Z=LA\Y;
    Un=UA\Z;
    U(:,i+1)=Un; %solution at each time
    U(GammaD1,i+1)=Ud(GammaD1);

end

