
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%computes stiffness matrix, right hand-side and solution U using P2 finite%
%elements  for the equation S*dU/dt-div(S*grad(U))=Q                      %
%inputs :                                                                 %
%         secondM : sink source function                                  %
%         s : storage coefficient                                         %
%         GammaD : Dirichlet's border                                     %
%         coord : nodes coordinates                                       %
%         nodes : connectivites                                           %
%         S :  hydraulic conductivity  or its decimal logarithm           %
%         Tmax : time of simulation                                       %
%         n : number of time steps                                        %
%         zD : values of P on Dirichlet border                          %
%         zN : values of Neumann border                                 %
%         boolean : 1 if we use logarithm of hydraulic conductivity       %
%                   0 otherwise                                           %
%outputs :                                                                %
%         U : time-dependent solution                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Abdellahi SOUEID AHMED, %
% Abderrahim JARDANI,     %
% Andr REVIL,            %
% 09/24/2012              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [U]=Time_dependentP2(S,s,secondM,GammaD,coord,nodes,zD,zN,Tmax,n,boolean)

global gg  %gravity constant
global rho % volumic mass

[ e, te ] = edgelist( coord, nodes);
test = findinterior( coord, e, te ); %interior nodes

GammaD1=(GammaD) &(~test) ; %Dirichlet's points
GammaN1=(~test)&(~GammaD1); % Neumann points

nbN=size(coord,1); %nombre total de noeuds
U=zeros(nbN,1);
f=U;
UNmann=U; %nbN nombre total de noeuds

N_unk=size(find(~GammaD1==1),1); %Number of unknowns
UNmann(~GammaD1)=(1:1:N_unk)';




qq=secondM(coord,0);
Ud=zeros(nbN,1);
%evaluates U on Dirichlet's borders
for j=1:nbN
    f(j)=qq(j,1);
    if GammaD1(j)
        Ud(j)=zD(j);
    end
end
for j=1:n+1
    U(:,j)=Ud(:);
end

%initialization of mass global matrix and stiffness global matrix
Mgl=sparse(N_unk,N_unk);  Kgl=sparse(N_unk,N_unk);

centers=center(coord,nodes);
if boolean==1
    kappa=(10.^(-S)/(9.82*1000));
    kappa=griddata(centers(:,1),centers(:,2),kappa(:),coord(:,1),coord(:,2),'nearest');
else
    kappa=S;
    kappa=griddata(centers(:,1),centers(:,2),kappa(:),coord(:,1),coord(:,2),'nearest');
end


%values os integration of shape shape function products
int1=[  2.00000000e-01  -3.33333333e-02  -3.33333333e-02   0.00000000e+00 -1.33333333e-01  -1.66533454e-16];
int2=[ -3.33333333e-02   2.00000000e-01  -3.33333333e-02   0.00000000e+00 0.00000000e+00  -1.33333333e-01];
int3=[ -3.33333333e-02  -3.33333333e-02   2.00000000e-01  -1.33333333e-01 0.00000000e+00  -1.66533454e-16];
int4=[  0.00000000e+00   0.00000000e+00  -1.33333333e-01   1.06666667e+00  5.33333333e-01   5.33333333e-01];
int5=[ -1.33333333e-01   0.00000000e+00   0.00000000e+00   5.33333333e-01 1.06666667e+00   5.33333333e-01];
int6= [ -1.66533454e-16  -1.33333333e-01  -1.66533454e-16   5.33333333e-01 5.33333333e-01   1.06666667e+00];

%loop over elements
for iel=1:size(nodes,1)
    %(j,k,l) : element en question
    j=nodes(iel,1); k=nodes(iel,2); l=nodes(iel,3); jk=nodes(iel,4);
    kl=nodes(iel,5); jl=nodes(iel,6);

    %UNmann(a)>0 if a is unknown
    jj=UNmann(j); kk=UNmann(k); ll=UNmann(l); jjkk=UNmann(jk); jjll=UNmann(jl); kkll=UNmann(kl);

    belem=[f(j) f(k) f(l) f(jk) f(kl) f(jl)];

    x21 = coord(k,1) - coord(j,1);
    x31 = coord(l,1) - coord(j,1);
    y21 = coord(k,2) - coord(j,2);
    y31 = coord(l,2) - coord(j,2);
    jacobian = x21*y31 - x31*y21;
    alpha = (x31^2 + y31^2)/jacobian;
    beta = -(x31*x21 + y31*y21)/jacobian;
    gamma = (x21^2 + y21^2)/jacobian;

    %elementary stiffness matrix
    Kel= alpha/6*[ 3  1  0 -4  0  0;1  3  0 -4  0  0; 0  0  0  0  0  0; ...
        -4 -4  0  8  0  0; 0  0  0  0  8 -8; 0  0  0  0 -8  8]...
        +beta/6*[ 6  1  1 -4  0 -4; 1  0 -1 -4  4  0; 1 -1  0  0  4 -4; ...
        -4 -4  0  8 -8  8; 0  4  4 -8  8 -8; -4  0 -4  8 -8  8]...
        +gamma/6*[ 3  0  1  0  0 -4; 0  0  0  0  0  0; 1  0  3  0  0 -4; ...
        0  0  0  8 -8  0; 0  0  0 -8  8  0; -4  0 -4  0  0  8];

    if boolean==1
        Kel=(10^(-S(iel))/(gg*rho))*Kel;
    else
        Kel=S(iel)*Kel;
    end

    %elementary mass matrix
    Mel= s(iel)*jacobian/360*[ 6 -1 -1  0 -4  0; -1  6 -1  0  0 -4;
        -1 -1  6 -4  0  0;0  0 -4 32 16 16; -4  0  0 16 32 16; 0 -4  0 16 16 32];


    if jj>0
        Kgl(jj,jj)=Kgl(jj,jj)+Kel(1,1);
        Mgl(jj,jj)=Mgl(jj,jj)+Mel(1,1);
        if (kk>0)
            Kgl(jj,kk)=Kgl(jj,kk)+Kel(1,2);
            Mgl(jj,kk)=Mgl(jj,kk)+Mel(1,2);
            Kgl(kk,jj)=Kgl(jj,kk);% matrice symetrique
            Mgl(kk,jj)=Mgl(jj,kk);% matrice symetrique
        end
        if (ll>0)
            Kgl(jj,ll)=Kgl(jj,ll)+Kel(1,3);
            Mgl(jj,ll)=Mgl(jj,ll)+Mel(1,3);
            Kgl(ll,jj)=Kgl(jj,ll);
            Mgl(ll,jj)=Mgl(jj,ll);
        end
        if (jjkk>0)
            Kgl(jj,jjkk)=Kgl(jj,jjkk)+Kel(1,4);
            Mgl(jj,jjkk)=Mgl(jj,jjkk)+Mel(1,4);
            Kgl(jjkk,jj)=Kgl(jj,jjkk);
            Mgl(jjkk,jj)=Mgl(jj,jjkk);
        end

        if (jjll>0)
            Kgl(jj,jjll)=Kgl(jj,jjll)+Kel(1,6);
            Mgl(jj,jjll)=Mgl(jj,jjll)+Mel(1,6);
            Kgl(jjll,jj)=Kgl(jj,jjll);
            Mgl(jjll,jj)=Mgl(jj,jjll);
        end

        if (kkll>0)
            Kgl(jj,kkll)=Kgl(jj,kkll)+Kel(1,5);
            Mgl(jj,kkll)=Mgl(jj,kkll)+Mel(1,5);
            Kgl(kkll,jj)=Kgl(jj,kkll);
            Mgl(kkll,jj)=Mgl(jj,kkll);
        end
    end


    if kk>0
        Kgl(kk,kk)=Kgl(kk,kk)+Kel(2,2);
        Mgl(kk,kk)=Mgl(kk,kk)+Mel(2,2);
        if ll>0
            Kgl(kk,ll)=Kgl(kk,ll)+Kel(2,3);
            Mgl(kk,ll)=Mgl(kk,ll)+Mel(2,3);
            Kgl(ll,kk)=Kgl(kk,ll);
            Mgl(ll,kk)=Mgl(kk,ll);
        end

        if jjkk>0
            Kgl(kk,jjkk)= Kgl(kk,jjkk)+Kel(2,4);
            Mgl(kk,jjkk)= Mgl(kk,jjkk)+Mel(2,4);
            Kgl(jjkk,kk)=Kgl(kk,jjkk);
            Mgl(jjkk,kk)=Mgl(kk,jjkk);
        end

        if jjll>0
            Kgl(kk,jjll)= Kgl(kk,jjll)+Kel(2,6);
            Mgl(kk,jjll)= Mgl(kk,jjll)+Mel(2,6);
            Kgl(jjll,kk)=Kgl(kk,jjll);
            Mgl(jjll,kk)=Mgl(kk,jjll);
        end

        if kkll>0
            Kgl(kk,kkll)= Kgl(kk,kkll)+Kel(2,5);
            Mgl(kk,kkll)= Mgl(kk,kkll)+Mel(2,5);
            Kgl(kkll,kk)=Kgl(kk,kkll);
            Mgl(kkll,kk)=Mgl(kk,kkll);
        end
    end



    if ll>0
        Kgl(ll,ll)=Kgl(ll,ll)+Kel(3,3);
        Mgl(ll,ll)=Mgl(ll,ll)+Mel(3,3);
        if jjkk>0
            Kgl(ll,jjkk)= Kgl(ll,jjkk)+Kel(3,4);
            Kgl(jjkk,ll)=Kgl(ll,jjkk);
            Mgl(ll,jjkk)= Mgl(ll,jjkk)+Mel(3,4);
            Mgl(jjkk,ll)=Mgl(ll,jjkk);
        end

        if jjll>0
            Kgl(ll,jjll)= Kgl(ll,jjll)+Kel(3,6);
            Kgl(jjll,ll)=Kgl(ll,jjll);
            Mgl(ll,jjll)= Mgl(ll,jjll)+Mel(3,6);
            Mgl(jjll,ll)=Mgl(ll,jjll);
        end

        if kkll>0
            Kgl(ll,kkll)= Kgl(ll,kkll)+Kel(3,5);
            Kgl(kkll,ll)=Kgl(ll,kkll);
            Mgl(ll,kkll)= Mgl(ll,kkll)+Mel(3,5);
            Mgl(kkll,ll)=Mgl(ll,kkll);
        end
    end


    if jjkk>0
        Kgl(jjkk,jjkk)=Kgl(jjkk,jjkk)+Kel(4,4);
        Mgl(jjkk,jjkk)=Mgl(jjkk,jjkk)+Mel(4,4);
        if (jjll>0)
            Kgl(jjkk,jjll)=Kgl(jjkk,jjll)+Kel(4,6);
            Kgl(jjll,jjkk)=Kgl(jjkk,jjll);% matrice symetrique
            Mgl(jjkk,jjll)=Mgl(jjkk,jjll)+Mel(4,6);
            Mgl(jjll,jjkk)=Mgl(jjkk,jjll);% matrice symetrique
        end
        if (kkll>0)
            Kgl(jjkk,kkll)=Kgl(jjkk,kkll)+Kel(4,5);
            Kgl(kkll,jjkk)=Kgl(jjkk,kkll);
            Mgl(jjkk,kkll)=Mgl(jjkk,kkll)+Mel(4,5);
            Mgl(kkll,jjkk)=Mgl(jjkk,kkll);
        end
    end

    if kkll>0
        Kgl(kkll,kkll)= Kgl(kkll,kkll)+Kel(5,5);
        Mgl(kkll,kkll)= Mgl(kkll,kkll)+Mel(5,5);

        if jjll>0
            Kgl(kkll,jjll)= Kgl(kkll,jjll)+Kel(6,5);
            Kgl(jjll,kkll)=Kgl(kkll,jjll);
            Mgl(kkll,jjll)= Mgl(kkll,jjll)+Mel(6,5);
            Mgl(jjll,kkll)=Mgl(kkll,jjll);
        end
    end


    if jjll>0
        Kgl(jjll,jjll)= Kgl(jjll,jjll)+Kel(6,6);
        Mgl(jjll,jjll)= Mgl(jjll,jjll)+Mel(6,6);
    end


end

%change size of global matrices to take account for boundary conditions
indices=find(GammaD1==1);
ind=setdiff([1:nbN] , indices);

AA=zeros(nbN,nbN);
for i=1:length(indices)
    AA(indices(i),:)=0;
    AA(indices(i),indices(i))=1;
end
com=1;
for i=1:nbN
    if (AA(i,:)==zeros(1,nbN))==ones(1,nbN)
        AA(i,ind)=Kgl(com,:);
        com=com+1;
    end
end

BB=zeros(nbN,nbN);
for i=1:length(indices)
    BB(indices(i),:)=0;
    BB(indices(i),indices(i))=1;
end
com=1;
for i=1:nbN
    if (BB(i,:)==zeros(1,nbN))==ones(1,nbN)
        BB(i,ind)=Mgl(com,:);
        com=com+1;
    end
end

%theta-scheme
theta=1;
tt=0; %initial time
dt = Tmax/n; %time step

A=BB+dt*theta*AA;
B=BB-dt*(1-theta)*AA;


[LA,UA]=lu(A);% LU factorization of A


Fn=zeros(nbN,1); %initialization of right-hand side

%loop over time
for i=1:n

    second_term=zeros(N_unk,1);
    tt = tt + dt; % time updating
    f=secondM(coord,tt);

    %loop over elements
    for iel=1:size(nodes,1)
        %(j,k,l) : element en question
        j=nodes(iel,1); k=nodes(iel,2); l=nodes(iel,3); jk=nodes(iel,4);
        kl=nodes(iel,5); jl=nodes(iel,6);

        %UNmann(a)>0 if a is unknown
        jj=UNmann(j); kk=UNmann(k); ll=UNmann(l); jjkk=UNmann(jk); jjll=UNmann(jl); kkll=UNmann(kl);

        belem=[f(j) f(k) f(l) f(jk) f(kl) f(jl)];

        x21 = coord(k,1) - coord(j,1);
        x31 = coord(l,1) - coord(j,1);
        y21 = coord(k,2) - coord(j,2);
        y31 = coord(l,2) - coord(j,2);
        jacobian = x21*y31 - x31*y21;
        alpha = (x31^2 + y31^2)/jacobian;
        beta = -(x31*x21 + y31*y21)/jacobian;
        gamma = (x21^2 + y21^2)/jacobian;

        %elementary stiffness matrix
        Kel= alpha/6*[ 3  1  0 -4  0  0;1  3  0 -4  0  0; 0  0  0  0  0  0; ...
            -4 -4  0  8  0  0; 0  0  0  0  8 -8; 0  0  0  0 -8  8]...
            +beta/6*[ 6  1  1 -4  0 -4; 1  0 -1 -4  4  0; 1 -1  0  0  4 -4; ...
            -4 -4  0  8 -8  8; 0  4  4 -8  8 -8; -4  0 -4  8 -8  8]...
            +gamma/6*[ 3  0  1  0  0 -4; 0  0  0  0  0  0; 1  0  3  0  0 -4; ...
            0  0  0  8 -8  0; 0  0  0 -8  8  0; -4  0 -4  0  0  8];

        if boolean==1
            Kel=(10^(-S(iel))/(gg*rho))*Kel;
        else
            Kel=S(iel)*Kel;
        end
        cste=jacobian/12;

        if jj>0
            second_term(jj)=second_term(jj)+cste*belem*int1';
        else
            if kk>0
                %parte du second memebe faisant intervenir les termes de la matrice
                %de rigidite
                second_term(kk)=second_term(kk)-U(j,i)*Kel(1,2);
            end
            if ll>0
                second_term(ll)=second_term(ll)-U(j,i)*Kel(1,3);
            end,
            if jjkk>0
                second_term(jjkk)=second_term(jjkk)-U(j,i)*Kel(1,4);
            end,
            if jjll>0
                second_term(jjll)=second_term(jjll)-U(j,i)*Kel(1,6);
            end,
            if kkll>0
                second_term(kkll)=second_term(kkll)-U(j,i)*Kel(1,5);
            end,
        end








        if kk>0
            second_term(kk)=second_term(kk)+cste*belem*int2';
        else
            if jj>0,
                second_term(jj)=second_term(jj)-U(k,i)*Kel(1,2);
            end
            if ll>0,
                second_term(ll)=second_term(ll)-U(k,i)*Kel(2,3);
            end
            if jjkk>0
                second_term(jjkk)=second_term(jjkk)-U(k,i)*Kel(2,4);
            end,
            if jjll>0
                second_term(jjll)=second_term(jjll)-U(k,i)*Kel(2,6);
            end,
            if kkll>0
                second_term(kkll)=second_term(kkll)-U(k,i)*Kel(2,5);
            end
        end


        if ll>0
            second_term(ll)=second_term(ll)+cste*belem*int3';
        else
            if jj>0,
                second_term(jj)=second_term(jj)-U(l,i)*Kel(1,3);
            end
            if kk>0,
                second_term(kk)=second_term(kk)-U(l,i)*Kel(2,3);
            end

            if jjkk>0
                second_term(jjkk)=second_term(jjkk)-U(l,i)*Kel(3,4);
            end,
            if jjll>0
                second_term(jjll)=second_term(jjll)-U(l,i)*Kel(3,6);
            end,
            if kkll>0
                second_term(kkll)=second_term(kkll)-U(l,i)*Kel(3,5);
            end

        end



        if jjkk>0
            second_term(jjkk)=second_term(jjkk)+cste*belem*int4';
        else
            if jj>0,
                second_term(jj)=second_term(jj)-U(jk,i)*Kel(4,1);
            end
            if kk>0,
                second_term(kk)=second_term(kk)-U(jk,i)*Kel(4,2);
            end

            if ll>0
                second_term(ll)=second_term(ll)-U(jk,i)*Kel(4,3);
            end,
            if jjll>0
                second_term(jjll)=second_term(jjll)-U(jk,i)*Kel(4,6);
            end,
            if kkll>0
                second_term(kkll)=second_term(kkll)-U(jk,i)*Kel(4,5);
            end

        end



        if kkll>0
            second_term(kkll)=second_term(kkll)+cste*belem*int5';
        else
            if jj>0,
                second_term(jj)=second_term(jj)-U(kl,i)*Kel(5,1);
            end
            if kk>0,
                second_term(kk)=second_term(kk)-U(kl,i)*Kel(5,2);
            end

            if ll>0
                second_term(ll)=second_term(ll)-U(kl,i)*Kel(5,3);
            end,
            if jjkk>0
                second_term(jjkk)=second_term(jjkk)-U(kl,i)*Kel(5,4);
            end,
            if jjll>0
                second_term(jjll)=second_term(jjll)-U(kl,i)*Kel(5,6);
            end

        end





        if jjll>0
            second_term(jjll)=second_term(jjll)+cste*belem*int6';
        else
            if jj>0,
                second_term(jj)=second_term(jj)-U(jl,i)*Kel(6,1);
            end
            if kk>0,
                second_term(kk)=second_term(kk)-U(jl,i)*Kel(6,2);
            end

            if ll>0
                second_term(ll)=second_term(ll)-U(jl,i)*Kel(6,3);
            end,
            if jjkk>0
                second_term(jjkk)=second_term(jjkk)-U(jl,i)*Kel(6,4);
            end,
            if kkll>0
                second_term(kkll)=second_term(kkll)-U(jl,i)*Kel(6,5);
            end

        end

        %in case we have Neumann contributions, we have to add its contributions
        if GammaN1(j)% in Neumann's border
            if test(k)==0,  %on boundaries
                %1D shape functions evaluation on  GammmaN
                second_term(jj)=second_term(jj)+Neumann_borderP2(coord(j,:),coord(k,:),zN(j),1);
                second_term(jjkk)=second_term(jjkk)+Neumann_borderP2(coord(j,:),coord(k,:),zN(j),2);
            end
            if test(l)==0,
                second_term(jj)=second_term(jj)+Neumann_borderP2(coord(j,:),coord(l,:),zN(j),1);
                second_term(jjll)=second_term(jjll)+Neumann_borderP2(coord(j,:),coord(l,:),zN(j),2);
            end
        end

        if GammaN1(k)
            if test(j)==0,
                second_term(kk)=second_term(kk)+Neumann_borderP2(coord(k,:),coord(j,:),zN(k),1);
            end
            if test(l)==0,
                second_term(kk)=second_term(kk)+Neumann_borderP2(coord(k,:),coord(l,:),zN(k),1);
                second_term(kkll)=second_term(kkll)+Neumann_borderP2(coord(k,:),coord(l,:),zN(k),2);
            end
        end

        if GammaN1(l)
            if test(j)==0,
                second_term(ll)=second_term(ll)+Neumann_borderP2(coord(l,:),coord(j,:),zN(l),1);
            end
            if test(k)==0,
                second_term(ll)=second_term(ll)+Neumann_borderP2(coord(l,:),coord(k,:),zN(l),1);
            end

        end
    end
    bb=zeros(nbN,1);%initialization of right-hand side
    bb(~GammaD1)=second_term;

    %values on Dirichlet borders are already known
    bb(GammaD1)=Ud(GammaD1);

    Fnp1=bb;
    F=dt*((1-theta)*Fn+theta*Fnp1);
    Fn=Fnp1;%updates theta-scheme

    %solves problem
    Y=B*U(:,i)+F;
    Z=LA\Y;
    Un=UA\Z;
    U(:,i+1)=Un; %solution at each time
    U(GammaD1,i+1)=Ud(GammaD1);


end
